"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.validateProjectConfig = exports.validateMtscConfig = exports.findMtscConfig = void 0;
const fs = require("fs");
const debugTools_1 = require("../helpers/debugTools");
const fileSystemHelpers_1 = require("../helpers/fileSystemHelpers");
const typeCheckHelpers_1 = require("../helpers/typeCheckHelpers");
const DEFAULT_CONFIG_NAME = "mtsc.json";
function findMtscConfig(path) {
    if ((0, typeCheckHelpers_1.isValidString)(path)) {
        (0, debugTools_1.debugLog)("Trying to open mtsc config", path);
        const filePath = (0, fileSystemHelpers_1.findJsonFile)(path, DEFAULT_CONFIG_NAME);
        return JSON.parse(fs.readFileSync(filePath, "utf8"));
    }
    try {
        (0, debugTools_1.debugLog)("Trying to find default mtsc config", path);
        (0, fileSystemHelpers_1.canAccessPath)(DEFAULT_CONFIG_NAME, true);
        return JSON.parse(fs.readFileSync(DEFAULT_CONFIG_NAME, "utf8"));
    }
    catch (_a) {
        (0, debugTools_1.debugLog)("Cannot find", DEFAULT_CONFIG_NAME);
    }
    return null;
}
exports.findMtscConfig = findMtscConfig;
function validateMtscConfig(config) {
    (0, debugTools_1.debugLog)("Validating mtsc config...");
    // tslint:disable-next-line:strict-type-predicates
    if (typeof config !== "object")
        throw new Error("Config isn't an object");
    assertTypeIfDefined("Config.Debug", config.debug, typeCheckHelpers_1.isValidBoolean);
    assertTypeIfDefined("Config.watch", config.watch, typeCheckHelpers_1.isValidBoolean);
    assertTypeIfDefined("Config.noEmit", config.noEmit, typeCheckHelpers_1.isValidBoolean);
    assertTypeIfDefined("Config.compiler", config.compiler, typeCheckHelpers_1.isValidString);
    const yarnWorkspaces = assertTypeIfDefined("Config.useYarnWorkspaces", config.useYarnWorkspaces, typeCheckHelpers_1.isValidBoolean);
    if (yarnWorkspaces) {
        assertTypeIfDefined("Config.projects", config.projects, typeCheckHelpers_1.isValidArray);
        config.projects = config.projects || [];
    }
    else {
        assertType("Config.projects", config.projects, typeCheckHelpers_1.isValidArray);
    }
    config.projects.forEach(validateProjectConfig);
    (0, debugTools_1.debugLog)("Mtsc config is valid!");
}
exports.validateMtscConfig = validateMtscConfig;
function validateProjectConfig(projectConfig) {
    if ((0, typeCheckHelpers_1.isValidString)(projectConfig)) {
        return;
    }
    if (!(0, typeCheckHelpers_1.isValidObject)(projectConfig)) {
        throw new Error("Project config is neither a valid object or a string");
    }
    assertType("ProjectConfig.path", projectConfig.path, typeCheckHelpers_1.isValidString);
    assertTypeIfDefined("ProjectConfig.compiler", projectConfig.compiler, typeCheckHelpers_1.isValidString);
    assertTypeIfDefined("ProjectConfig.noEmit", projectConfig.noEmit, typeCheckHelpers_1.isValidBoolean);
}
exports.validateProjectConfig = validateProjectConfig;
function assertTypeIfDefined(description, value, match) {
    if (value !== undefined && value !== null && !match(value)) {
        const matchType = match.name.substr("isValid".length);
        throw new Error(`${description} isn't a valid ${matchType}. The given value is: ${value}`);
    }
    return true;
}
function assertType(description, value, match) {
    if (!match(value)) {
        const matchType = match.name.substr("isValid".length);
        throw new Error(`${description} isn't a valid ${matchType}. The given value is: ${value}`);
    }
    return true;
}
