"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProjectsWatcher = void 0;
const moment = require("moment");
const cluster = require("cluster");
const debugTools_1 = require("../helpers/debugTools");
const COMPILER_STARTUP = "Starting compilation in watch mode...";
const COMPILER_DONE = "Compilation complete. Watching for file changes.";
const COMPILER_BUILDING = "File change detected. Starting incremental compilation...";
class ProjectsWatcher {
    constructor() {
        this.projectWorkers = new Map();
        this.handleWorkerMessage = (worker, stateUpdate) => {
            (0, debugTools_1.debugLog)(`Worker ${worker.id} says: `, stateUpdate.projectState);
            this.projectWorkers.get(worker).projectState = stateUpdate;
            if (stateUpdate.projectState === "COMPILING") {
                if (this.globalState !== "COMPILING")
                    logMessage(COMPILER_BUILDING);
                this.globalState = "COMPILING";
                (0, debugTools_1.debugLog)("Global state now: COMPILING");
                return;
            }
            // ProjectState from here on is [COMPLETE]
            if (this.globalState !== "COMPILING") {
                // We didn't know that we were apparently [COMPILING]? Let's pretend we did :)
                logMessage(COMPILER_BUILDING);
                this.globalState = "COMPILING";
                (0, debugTools_1.debugLog)("Global state now: COMPILING");
            }
            process.stdout.write(this.getLastResults());
            logMessage(COMPILER_DONE);
            this.globalState = "COMPLETE";
            if (this.getMostActiveWorkerState() === "COMPILING") {
                // Some project is still [COMPILING]?
                logMessage(COMPILER_BUILDING);
                this.globalState = "COMPILING";
            }
            (0, debugTools_1.debugLog)("Global state now: ", this.globalState);
        };
        cluster.addListener("message", this.handleWorkerMessage);
        this.globalState = "COMPILING";
        logMessage(COMPILER_STARTUP);
    }
    addWorker(projectSettings) {
        (0, debugTools_1.debugLog)("Creating project compiler for", projectSettings.path);
        const workerInfo = { projectSettings, projectState: { lastResult: "", projectState: "COMPLETE" } };
        this.projectWorkers.set(cluster.fork({ projectSettings: JSON.stringify(projectSettings) }), workerInfo);
    }
    startCompilations() {
        const workerInstruction = "START";
        for (const [worker] of this.projectWorkers) {
            worker.send(workerInstruction);
        }
    }
    getMostActiveWorkerState() {
        for (const [, workerInfo] of this.projectWorkers) {
            const workerState = workerInfo.projectState;
            if (workerState.projectState === "COMPILING")
                return "COMPILING";
        }
        return "COMPLETE";
    }
    getLastResults() {
        const results = [];
        this.projectWorkers.forEach(({ projectState }) => projectState.lastResult && results.push(projectState.lastResult));
        return results.join("\n");
    }
}
exports.ProjectsWatcher = ProjectsWatcher;
function getTimestamp() {
    return moment().format("HH:mm:ss");
}
function logMessage(message) {
    process.stdout.write(`${getTimestamp()} - ${message}\n`);
}
